mod error;
// mod metrics;
mod server;

pub use error::Result;
use tokio::net::TcpListener;

use std::{net::SocketAddr, path::PathBuf};

use tracing_subscriber::{layer::SubscriberExt, util::SubscriberInitExt};

/// Name of the directory where static sites are stored inside the data directory
const STATIC_DIR_NAME: &str = "static";
const STATIC_ROOT_NAME: &str = "_root";

const REDIRECTS: [(&str, &str); 6] = [
    ("/github", "https://github.com/ChewingBever"),
    ("/gitea", "https://git.rustybever.be/Chewing_Bever"),
    ("/gitlab", "https://gitlab.com/Chewing_Bever"),
    ("/codeberg", "https://codeberg.org/Chewing_Bever"),
    ("/matrix", "https://matrix.to/#/@jef:rustybever.be"),
    ("/aur", "https://aur.archlinux.org/account/Chewing_Bever"),
];

#[derive(Clone)]
pub struct Context {
    static_dir: PathBuf,
    tmp_dir: PathBuf,
}

#[tokio::main]
async fn main() {
    // Enable tracing
    tracing_subscriber::registry()
        .with(tracing_subscriber::EnvFilter::new(
            std::env::var("RUST_LOG").unwrap_or_else(|_| "site=debug,tower_http=debug".into()),
        ))
        .with(tracing_subscriber::fmt::layer())
        .init();

    // Get required variables from env vars
    let api_key = std::env::var("API_KEY").expect("No API_KEY was provided.");
    let data_dir = PathBuf::from(std::env::var("DATA_DIR").expect("No DATA_DIR was provided."));
    let static_dir = data_dir.join(STATIC_DIR_NAME);

    std::fs::create_dir_all(&static_dir).unwrap();

    let state = Context {
        static_dir,
        tmp_dir: std::env::temp_dir(),
    };

    // Initialize metrics
    // let recorder_handle = metrics::setup_metrics_recorder();
    let app = server::app(state, &api_key, &REDIRECTS);

    let addr = SocketAddr::from(([0, 0, 0, 0], 3000));
    let listener = TcpListener::bind(addr).await.unwrap();
    tracing::debug!("listening on {}", addr);
    axum::serve(listener, app).await.unwrap();
}
